#include "civedit.h"
#include "draw.h"
#include "events.h"
#include "tiles.h"

#include "unit.h"
#include "map.h"
/*
#include "genlist.h"
*/

#include <stdio.h>
#include <string.h>

int is_solid_bg=TRUE;
int is_startpos_units=TRUE;


/*****************************************************************************
initialize the pixmaps and graphics contexts
*****************************************************************************/
void my_init(GtkWidget *widget)
 {
  /* get white and use it as transparent color. */
  gdk_color_white(gdk_window_get_colormap(widget->window), &white); 
  pix_tiles=gdk_pixmap_create_from_xpm(widget->window, &mask_tiles,
				       &white, "../data/tiles.xpm");  
  pix_units=gdk_pixmap_create_from_xpm(widget->window, &mask,
				  &white, "../data/units.xpm");  

  /* get the size of the tiles */
  gdk_window_get_size(pix_tiles,&TILE_WIDTH,&TILE_HEIGHT);
  TILE_WIDTH/=20;TILE_HEIGHT/=21;

  /* copy the masks into the gc */
  gc_tiles=gdk_gc_new(widget->window);
  gdk_gc_set_clip_mask(gc_tiles, mask_tiles);
  gc=gdk_gc_new(widget->window);
  gdk_gc_set_clip_mask(gc, mask);

 } /* my_init() */


/*****************************************************************************
draw a unit: first draw the appropriate flag, then the unit itself
*****************************************************************************/
void draw_pixmap(GtkWidget *widget, int unit, int race, int x, int y)
 {
  int tmpX;
  int tmpY;

  if(!is_solid_bg)
   {
    draw_tile(widget, x/TILE_WIDTH, y/TILE_HEIGHT, NULL);
    tmpX=race%16;
    tmpY=14;
    gdk_gc_set_clip_origin(gc_tiles,(x/TILE_WIDTH-tmpX)*TILE_WIDTH,
			   (y/TILE_HEIGHT-tmpY)*TILE_HEIGHT);
    gdk_draw_pixmap(pix_screen, gc_tiles, pix_tiles,
		    (race%16)*TILE_WIDTH, 14*TILE_HEIGHT,
		    (x/TILE_WIDTH)*TILE_WIDTH,(y/TILE_HEIGHT)*TILE_HEIGHT, 
		    TILE_WIDTH, TILE_HEIGHT);
   }
  else
    gdk_draw_pixmap(pix_screen, widget->style->white_gc, pix_tiles,
		    (race%16)*TILE_WIDTH, 14*TILE_HEIGHT,
		    (x/TILE_WIDTH)*TILE_WIDTH,(y/TILE_HEIGHT)*TILE_HEIGHT, 
		    TILE_WIDTH, TILE_HEIGHT);

  if(is_startpos_units)
   {
    tmpX=unit_types[unit].graphics%20;
    tmpY=unit_types[unit].graphics/20;
    gdk_gc_set_clip_origin(gc,(x/TILE_WIDTH-tmpX)*TILE_WIDTH,
                           (y/TILE_HEIGHT-tmpY)*TILE_HEIGHT);
    gdk_draw_pixmap(pix_screen, gc, pix_units,
		     tmpX*TILE_WIDTH, tmpY*TILE_HEIGHT, 
		    (x/TILE_WIDTH)*TILE_WIDTH,(y/TILE_HEIGHT)*TILE_HEIGHT, 
		     TILE_WIDTH, TILE_HEIGHT);
   }
 } /* draw_pixmap() */


/*****************************************************************************
draw the units: first draw settlers on all starting positions (then all units)
*****************************************************************************/
void draw_units(GtkWidget *widget, int x, int y, int w, int h)
{
/*  struct genlist_iterator iterator;
  struct unit *punit; */
  int i;

  for(i=0;i<R_LAST;i++)
    if((map.start_positions[i].x!=0) ||
      (map.start_positions[i].y!=0))
      draw_pixmap(widget,U_SETTLERS,i,
                  map.start_positions[i].x*TILE_WIDTH,
                  map.start_positions[i].y*TILE_HEIGHT);

/*
  for(i=0; i<game.nplayers; i++)
    {
     genlist_iterator_init(&iterator,
                          (struct genlist *) &game.players[i].units, 0); 
     for(; ITERATOR_PTR(iterator); ITERATOR_NEXT(iterator)) 
      {
	punit=(struct unit *)ITERATOR_PTR(iterator);
	if((punit->x >= x) &&(punit->y >= y) &&
	   (punit->x <=(x+w+1)) &&(punit->y <=(y+h+1)))
	 {
	  draw_pixmap(widget, punit->type, game.players[punit->owner].race,
		       punit->x*TILE_WIDTH, punit->y*TILE_HEIGHT);  
	 }
      }
    }
*/

 } /* draw_units() */


/*****************************************************************************
redraw the whole map screen by generating a fake expose event
*****************************************************************************/
void refresh_screen()
{
  GdkEventExpose tmp;

  tmp.area.x=0;
  tmp.area.y=0;
  tmp.area.width=map.xsize*TILE_WIDTH;
  tmp.area.height=map.ysize*TILE_HEIGHT;

  expose(GTK_WIDGET(da), &tmp);
} /* refresh_screen() */


void destroy_me_callback( GtkWidget *w, gpointer data)
{
  gtk_widget_destroy(w);
}


/*****************************************************************************
pop up information about the current tile
*****************************************************************************/
void popit(GdkEventButton *ev, int x, int y)
 {
  char s[64];
  struct tile *ptile;
  GtkWidget *item, *p=gtk_menu_new();

  gtk_signal_connect_after(GTK_OBJECT(p),"hide",
			   GTK_SIGNAL_FUNC(destroy_me_callback),NULL);

  ptile=map_get_tile(x, y);
  sprintf(s, "Terrain: %s", map_get_tile_info_text(x, y));

  item=gtk_menu_item_new_with_label(s);
  gtk_widget_set_sensitive(item,FALSE);
  gtk_menu_append(GTK_MENU(p),item);

  if(ptile->special&S_HUT) {
    item=gtk_menu_item_new_with_label("Minor Tribe Village");
    gtk_widget_set_sensitive(item,FALSE);
    gtk_menu_append(GTK_MENU(p),item);
  }

/*
  if((pcity=map_get_city(xtile, ytile))) {
    sprintf(s, "City: %s(%s)", pcity->name, 
            get_race_name(game.players[pcity->owner].race));
    item=gtk_menu_item_new_with_label(s);
    gtk_widget_set_sensitive(item,FALSE);
    gtk_menu_append(GTK_MENU(p),item);
  }
*/

 if(ptile->special&S_ROAD || ptile->special&S_IRRIGATION ||
     ptile->special&S_RAILROAD || ptile->special&S_MINE ||
     ptile->special&S_FORTRESS) {
    sprintf(s, "Infrastructure: ");
    if(ptile->special&S_ROAD)
      strcat(s, "Road/");
    if(ptile->special&S_RAILROAD)
      strcat(s, "Railroad/");
    if(ptile->special&S_IRRIGATION)
      strcat(s, "Irrigation/");
    if(ptile->special&S_MINE)
      strcat(s, "Mine/");
    if(ptile->special&S_FORTRESS)
      strcat(s, "Fortress/");
    *(s+strlen(s)-1)='\0';
    item=gtk_menu_item_new_with_label(s);
    gtk_widget_set_sensitive(item,FALSE);
    gtk_menu_append(GTK_MENU(p),item);
  }

  gtk_widget_show_all(p);
  gtk_menu_popup(GTK_MENU(p),NULL,NULL,NULL,NULL,ev->button,ev->time);

 } /* popit() */


/*****************************************************************************
toggle the solid unit background on and off
*****************************************************************************/
void solid_bg()
 {
  is_solid_bg=(is_solid_bg==FALSE);
  draw_units(GTK_WIDGET(da), 0, 0, map.xsize*TILE_WIDTH, map.ysize*TILE_HEIGHT);
  refresh_screen();
 } /* solid_bg() */


/*****************************************************************************
toggle the drawing of units on and off
*****************************************************************************/
void startpos_units()
 {
  is_startpos_units=(is_startpos_units==FALSE);
  draw_units(GTK_WIDGET(da), 0, 0, map.xsize*TILE_WIDTH, map.ysize*TILE_HEIGHT);
  refresh_screen();
 } /* solid_bg() */
