#include "menu.h"
#include "main.h"
#include "civ.h"
#include "tiles.h"
#include "draw.h"

#include "registry.h"
#include "game.h"
#include "map.h"

#include <string.h>

GtkWidget *odialog=NULL;
GtkWidget *sdialog=NULL;

GtkWidget *spinnerx;
GtkWidget *spinnery;


/* File menu */
MyMenuData menu_file_names[8]=
 {
  {"File",             (MenuFunction) NULL},
  {"New",              (MenuFunction) new_file},
  {"Open",             (MenuFunction) open_file},
  {"Save",             (MenuFunction) save_file},
  {"Save as...",       (MenuFunction) save_file_as},
  {(char *) SEPARATOR, (MenuFunction) NULL},
  {"Quit",             (MenuFunction) gtk_main_quit},
  {NULL, NULL}
 };

/* Edit menu */
MyMenuData menu_edit_names[3]=
 {
  {"Edit",                 (MenuFunction) NULL},
  {"Map and game options", (MenuFunction) create_settings},
  {NULL, NULL}
 };

/* View menu */
MyMenuData menu_view_names[7]=
 {
  {"View",                     (MenuFunction) NULL},
  {"Show toolbox",             (MenuFunction) show_tools},
  {(char *) SEPARATOR,         (MenuFunction) NULL},
  {"\tShow tooltips",          (MenuFunction) show_tips},
  {"\tSolid unit background",  (MenuFunction) solid_bg},
  {"\tStart position units",   (MenuFunction) startpos_units},
  {NULL, NULL}
 };


/****************************************************************************
create all menus
*****************************************************************************/
void create_menus(GtkWidget *widget)
 {
  GtkWidget *menu_bar;

  menu_bar=gtk_menu_bar_new();
  create_menu(menu_bar, menu_file_names);
  create_menu(menu_bar, menu_edit_names);
  create_menu(menu_bar, menu_view_names);

  gtk_box_pack_start(GTK_BOX(widget), menu_bar, FALSE, FALSE, 2);
  gtk_widget_show(menu_bar);
} /* create_menus() */


/****************************************************************************
create a menu from the appropriate MenuData structure
*****************************************************************************/
void create_menu(GtkWidget *menu_bar, MyMenuData *data)
 {
  GtkWidget *menu;
  GtkWidget *menu_item;
  GtkWidget *root_menu;
  gint i;

  menu=gtk_menu_new();
  root_menu=gtk_menu_item_new_with_label(data[0].name);
  gtk_widget_show(root_menu);

  for(i=1; data[i].name != NULL; i++)
   {
    if(data[i].name==(char *)SEPARATOR)
     {
      menu_item = gtk_menu_item_new();
     }
    else
     {
      if(data[i].name[0]=='\t')
       {
	menu_item=gtk_check_menu_item_new_with_label(data[i].name+1);
	gtk_check_menu_item_set_show_toggle(GTK_CHECK_MENU_ITEM(menu_item),
        				    TRUE);
	gtk_check_menu_item_set_state(GTK_CHECK_MENU_ITEM(menu_item),TRUE);
       }
      else
	menu_item=gtk_menu_item_new_with_label(data[i].name);
      if(data[i].func != NULL)
       {
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			   GTK_SIGNAL_FUNC(data[i].func), NULL);
       }
     }
    gtk_menu_append(GTK_MENU(menu), menu_item);
    gtk_widget_show(menu_item);
   }

  gtk_menu_item_set_submenu(GTK_MENU_ITEM(root_menu), menu);
  gtk_menu_bar_append(GTK_MENU_BAR(menu_bar), root_menu);

 } /* create_menu() */


/****************************************************************************
button callbacks for mapsize dialog
*****************************************************************************/
void mapsize_ok(GtkWidget *widget, GtkWidget *dialog)
 {
  int x,y;

  x=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinnerx));
  y=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinnery));

  gtk_widget_destroy(widget);
  
  create_new_map(x,y);

  draw_everything();
 } /* settings_ok()*/

void mapsize_cancel(GtkWidget *widget, GtkWidget *dialog)
 {
  if(dialog!=NULL)
    gtk_widget_destroy(dialog);
  else
    gtk_widget_destroy(widget);
 } /* settings_cancel*/


/****************************************************************************
create a new game with default values
*****************************************************************************/
void new_file(void)
 {
  GtkWidget *mapsizewin;
  GtkWidget *table;
  GtkWidget *vbox;
  GtkWidget *hbox;
  GtkWidget *button;
  GtkWidget *label;
  GtkAdjustment *adj;

  mapsizewin=gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_signal_connect(GTK_OBJECT(mapsizewin), "delete_event",
		     GTK_SIGNAL_FUNC(mapsize_cancel), NULL);
  gtk_window_set_title(GTK_WINDOW(mapsizewin), "Create new map");

  vbox=gtk_vbox_new(FALSE, 5);
  gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
  gtk_container_add(GTK_CONTAINER(mapsizewin), vbox);

  table=gtk_table_new(2, 3, FALSE);
  gtk_box_pack_start(GTK_BOX(vbox), table, TRUE, TRUE, 0);

  vbox=gtk_vbox_new(FALSE, 0);
  gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
  gtk_table_attach(GTK_TABLE(table), vbox, 0, 1, 0, 1, 
		   GTK_FILL, GTK_FILL, 5, 5);

  label=gtk_label_new("xsize");
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
  gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);

  adj=(GtkAdjustment *)gtk_adjustment_new(MAP_DEFAULT_WIDTH,
					  MAP_MIN_WIDTH, MAP_MAX_WIDTH,
					  1.0, 5.0, 0.0);
  spinnerx=gtk_spin_button_new(adj, 0, 0);
  gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(spinnerx), TRUE);
  gtk_spin_button_set_shadow_type(GTK_SPIN_BUTTON(spinnerx),
				  GTK_SHADOW_ETCHED_IN);
  gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spinnerx), TRUE);
  gtk_tooltips_set_tip(tooltips, spinnerx, "Map width in squares", NULL);
  gtk_box_pack_start (GTK_BOX (vbox), spinnerx, FALSE, TRUE, 0);


  vbox=gtk_vbox_new(FALSE, 0);
  gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
  gtk_table_attach(GTK_TABLE(table), vbox, 1, 2, 0, 1, 
		   GTK_FILL, GTK_FILL, 5, 5);

  label=gtk_label_new("ysize");
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
  gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);

  adj=(GtkAdjustment *)gtk_adjustment_new(MAP_DEFAULT_HEIGHT,
					  MAP_MIN_HEIGHT, MAP_MAX_HEIGHT,
					  1.0, 5.0, 0.0);
  spinnery=gtk_spin_button_new(adj, 0, 0);
  gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(spinnery), TRUE);
  gtk_spin_button_set_shadow_type(GTK_SPIN_BUTTON(spinnery),
				  GTK_SHADOW_ETCHED_IN);
  gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spinnery), TRUE);
  gtk_tooltips_set_tip(tooltips, spinnery, "Map height in squares", NULL);
  gtk_box_pack_start (GTK_BOX (vbox), spinnery, FALSE, TRUE, 0);
  
  hbox=gtk_hseparator_new();
  gtk_table_attach(GTK_TABLE(table), hbox, 0, 2, 1, 2,
		   GTK_FILL, GTK_FILL, 0, 0);

  button=gtk_button_new_with_label("OK");
  gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
			    GTK_SIGNAL_FUNC(mapsize_ok),
			    GTK_OBJECT(mapsizewin));
  gtk_table_attach(GTK_TABLE(table), button, 0, 1, 2, 3,
		   GTK_FILL, GTK_FILL, 5, 5);

  button=gtk_button_new_with_label("Cancel");
  gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
			    GTK_SIGNAL_FUNC(mapsize_cancel),
			    GTK_OBJECT(mapsizewin));
  gtk_table_attach(GTK_TABLE(table), button, 1, 2, 2, 3,
		   GTK_FILL, GTK_FILL, 5, 5);

  gtk_widget_show_all(GTK_WIDGET(mapsizewin));
  gtk_window_set_policy(GTK_WINDOW(mapsizewin), FALSE, FALSE, FALSE);

 } /* new_file() */


/****************************************************************************
button callbacks for file dialogs
*****************************************************************************/
void fs_cancel(GtkWidget *widget, GtkFileSelection * fs)
 {
  gtk_file_selection_set_filename(GTK_FILE_SELECTION(fs), "");
  gtk_widget_hide(GTK_WIDGET(fs));
 } /* open_cancel() */


void open_ok(GtkWidget *widget, GtkFileSelection *fs)
 {
  gint i;

  load_game(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));
  gtk_widget_hide(GTK_WIDGET(fs));

  /* clear all specials from the map as we don't know what to do with them */
  clear_rect_specials(0, 0, map.xsize, map.ysize);

  map.generator=0;
  game.nplayers=0;
  game.scenario=2; /* default to tilemap + startpositions */
  /* default to no start positions if at least 1 is missing */
  for(i=0;i<R_LAST;i++)
    if((map.start_positions[i].x==0) &&
       (map.start_positions[i].y==0))
      game.scenario=3;

  draw_everything();
 } /* open_ok() */


void save_ok(GtkWidget *widget, GtkFileSelection *fs)
 {
  save_game(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));
  gtk_widget_hide(GTK_WIDGET(fs));
 } /* save_ok() */


/****************************************************************************
open a freeciv save game
*****************************************************************************/
void open_file()
 {

  if(odialog==NULL)
   {
    odialog = gtk_file_selection_new("Open file...");
    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(odialog)->cancel_button),
		       "clicked",(GtkSignalFunc) fs_cancel, odialog);
    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(odialog)->ok_button),
		       "clicked",(GtkSignalFunc) open_ok, odialog);
   }

  gtk_widget_show(odialog);
 } /* open_file() */


/****************************************************************************
save the map as the current freeciv save file
*****************************************************************************/
void save_file()
 {
  if((sdialog==NULL) || (strlen(gtk_file_selection_get_filename
      (GTK_FILE_SELECTION(sdialog)))==0))
    save_file_as();
  else
    save_game(gtk_file_selection_get_filename(GTK_FILE_SELECTION(sdialog)));
 } /* save_file() */


/****************************************************************************
save a freeciv save game
*****************************************************************************/
void save_file_as()
 {
  if(sdialog==NULL)
   {
    sdialog = gtk_file_selection_new("Save file as...");
    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(sdialog)->cancel_button),
		       "clicked",(GtkSignalFunc) fs_cancel, sdialog);
    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(sdialog)->ok_button),
		       "clicked",(GtkSignalFunc) save_ok, sdialog);
   }

  gtk_widget_show(sdialog);
 } /* save_file_as() */


/*
void do_help(GtkWidget *widget, char *string)
 {
  GtkWidget *button;
  GtkWidget *label;
  GtkWidget *dialog;

  dialog=gtk_dialog_new();
  gtk_window_set_title(GTK_WINDOW(dialog), "Help");

  button=gtk_button_new_with_label("OK");
  label=gtk_label_new(string);
  
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), button,
		     TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), label,
		     FALSE, TRUE, 10);  

  gtk_signal_connect(GTK_OBJECT(button), "clicked",
		     (GtkSignalFunc) help_close, dialog);
 
  gtk_widget_show(button);
  gtk_widget_show(label);
  gtk_widget_show(dialog);
 }


void help_close(GtkWidget *widget, GtkWidget *dialog)
 {
  gtk_widget_destroy(dialog);
 }

*/
