#include "tiles.h"
#include "main.h"
#include "draw.h"

#include "map.h"
#include "game.h"
/*
#include "city.h"
*/
/* from mapgen.c */
void init_workmap(void);

#include <stdlib.h>


int terrain_to_tile_map[13]= {
  ARCTIC_TILES, DESERT_TILES, FOREST_TILES, GRASSLAND_TILES,
  HILLS_TILES, JUNGLE_TILES, MOUNTAINS_TILES, OCEAN_TILES,
  PLAINS_TILES, RIVER_TILES, SWAMP_TILES, TUNDRA_TILES
};


/*****************************************************************************
draws all tiles and units onto the offscreen pixmap, then refreshes the screen
*****************************************************************************/
void draw_everything()
 {
  gint x,y;
  /* How can scrolled window get scrollbars automatically? -> hack: resize! */
  gtk_drawing_area_size(da, map.xsize*TILE_WIDTH, map.ysize*TILE_HEIGHT);
  gdk_window_get_size(win->window,&x,&y);
  gdk_window_resize(win->window,x,y-1);

  /* if we already have an off-screen pixmap, dump it. then make a new one */
  if (pix_screen)
    gdk_pixmap_unref(pix_screen);

  pix_screen=gdk_pixmap_new(win->window,
                            map.xsize*TILE_WIDTH, map.ysize*TILE_HEIGHT, -1);

  /* draw all terrain tiles on the off-screen pixmap */
  for(x=0;x<=map.xsize;x++)
   {
    for(y=0;y<=map.ysize;y++)
     {
      draw_tile(GTK_WIDGET(da),x,y, NULL); 
     }
/*
      draw_tile(GTK_WIDGET(da),x,y, &city_to_draw_list);
     }
    draw_cities(GTK_WIDGET(da), &city_to_draw_list);
*/
   }

  /* draw the units as well */
  draw_units(GTK_WIDGET(da), 0, 0, map.xsize, map.ysize);

  refresh_screen();
 } /* draw_everything() */


/****************************************************************************
create a new game with default values
*****************************************************************************/
void create_new_map(int x, int y)
 {
  int i;

  /* reset the map: delete the tiles*/
  if(map.tiles!=NULL)
    free(map.tiles);

  game_init();
  game.scenario=3; /* default to type 3 senario (w/o start positions) */
  for(i=0;i<R_LAST;i++)
   {
    map.start_positions[i].x=0;
    map.start_positions[i].y=0;
   }
  map.xsize=x;
  map.ysize=y;
  init_workmap(); /* now create the map */

  /* draw the tiles: arctic in the polar region, ocean everywhere else */
  for(x=0;x<map.xsize;x++)
    for(y=0;y<map.ysize;y++)
      if ((y==0) || (y==map.ysize-1))
	map_set_terrain(x, y, T_ARCTIC);
      else
	map_set_terrain(x,y,T_OCEAN);

  draw_everything();
 } /* create_new_map() */


/***************************************************************************
draw the terrain and specials of the tile at(x,y)
****************************************************************************/
void draw_tile(GtkWidget *widget, gint16 x, gint16 y, struct genlist *list)
 {
  int ttype;
  int tmpX, tmpY;
  int tileno;
  int den_y=map.ysize*.24;
  int ttype_north, ttype_south, ttype_east, ttype_west;
  int ttype_north_east, ttype_south_east, ttype_south_west, ttype_north_west;
  int tspecial, tspecial_north, tspecial_south, tspecial_east, tspecial_west;
  struct tile *ptile;
/*  struct city *pcity;
*/

  x=map_adjust_x(x);
  y=map_adjust_y(y);
  
  ptile=map_get_tile(x, y);
  
/*  if(pcity=map_get_city(x, y))
    {
      if(list != NULL)
	draw_city(widget, pcity, list);
      return;
    }
*/

  /* first, get the correct terrain tile to draw */
  ttype=map.tiles[x+(y*map.xsize)].terrain;
  tileno=terrain_to_tile_map[ttype];
  
  ttype_north=map_get_terrain(x, y-1);
  ttype_south=map_get_terrain(x, y+1);
  ttype_west=map_get_terrain(x-1, y);
  ttype_east=map_get_terrain(x+1, y);
  ttype_north_east=map_get_terrain(x+1, y-1);
  ttype_south_east=map_get_terrain(x+1, y+1);
  ttype_south_west=map_get_terrain(x-1, y+1);
  ttype_north_west=map_get_terrain(x-1, y-1);
  tspecial=map_get_special(x, y);

  switch(ttype)
   {
    case T_OCEAN:
      tileno+=(ttype_north!=T_OCEAN) ? 1 : 0;
      tileno+=(ttype_east!=T_OCEAN)  ? 2 : 0;
      tileno+=(ttype_south!=T_OCEAN) ? 4 : 0;
      tileno+=(ttype_west!=T_OCEAN)  ? 8 : 0;
    break;

    case T_RIVER:
      tileno=RIVER_TILES;
      tileno+=(ttype_north==T_RIVER || ttype_north==T_OCEAN) ? 1 : 0;
      tileno+=(ttype_east==T_RIVER  || ttype_east== T_OCEAN) ? 2 : 0;
      tileno+=(ttype_south==T_RIVER || ttype_south==T_OCEAN) ? 4 : 0;
      tileno+=(ttype_west==T_RIVER  || ttype_west== T_OCEAN) ? 8 : 0;
    break;

    case T_MOUNTAINS:
    case T_HILLS:
    case T_FOREST:
      if(ttype_west==ttype && ttype_east==ttype)
        tileno+=2;
      else if(ttype_west==ttype)
        tileno+=3;
      else if(ttype_east==ttype)
        tileno++;
    break;

    case T_GRASSLAND:
    case T_DESERT:
    case T_ARCTIC:
    case T_JUNGLE:
    case T_PLAINS:
    case T_SWAMP:
    case T_TUNDRA:
      tileno+=(ttype_north!=ttype) ? 1 : 0;
      tileno+=(ttype_east!=ttype)  ? 2 : 0;
      tileno+=(ttype_south!=ttype) ? 4 : 0;
      tileno+=(ttype_west!=ttype)  ? 8 : 0;
    break;
   } /* switch */

  /* if we are on Earth, place the denmark tiles at the appropriate location */
  if(map.is_earth && x>=34 && x<=36 && y>=den_y && y<=den_y+ 1) 
    tileno=(y-den_y)*3+x-34+DENMARK_TILES;

  tmpX=tileno%20;
  tmpY=tileno/20;

  if(genlist_size((struct genlist *) &(ptile->units.list)) == 0)
   {
    gdk_draw_pixmap(pix_screen, widget->style->white_gc, pix_tiles,
		     tmpX*TILE_WIDTH, tmpY*TILE_HEIGHT,
		     x*TILE_WIDTH,y*TILE_HEIGHT, TILE_WIDTH, TILE_HEIGHT);
  
    if(ttype==T_OCEAN)
     {
      tileno=CORNER_TILES-1;
      if((ttype_north==T_OCEAN && ttype_east==T_OCEAN && 
         ttype_north_east!=T_OCEAN))
        tileno+=1;
      if((ttype_east==T_OCEAN && ttype_south==T_OCEAN && 
         ttype_south_east!=T_OCEAN))
        tileno+=2;
      if((ttype_south==T_OCEAN && ttype_west==T_OCEAN && 
         ttype_south_west!=T_OCEAN))
        tileno+=4;
      if((ttype_north==T_OCEAN && ttype_west==T_OCEAN && 
         ttype_north_west!=T_OCEAN))
        tileno+=8;

      if(tileno!=CORNER_TILES-1)
        draw_special(widget, x, y, tileno);

      if(ttype_north==T_RIVER)
        draw_special(widget, x, y, OUTLET_TILES+0);
      if(ttype_west==T_RIVER)
        draw_special(widget, x, y, OUTLET_TILES+1);
      if(ttype_south==T_RIVER)
        draw_special(widget, x, y, OUTLET_TILES+2);
      if(ttype_east==T_RIVER)
        draw_special(widget, x, y, OUTLET_TILES+3);
     } /* if(T_OCEAN) */


    /* now draw the specials(infrastructure) on top of the terrain tile */
    if(tspecial & S_IRRIGATION)
      draw_special(widget, x, y, IRRIGATION_TILE);
    else if(tspecial & S_MINE)
     {
      if(ttype==T_HILLS || ttype==T_MOUNTAINS)
	draw_special(widget, x, y, HILLMINE_TILE);
      else /* desert */
	draw_special(widget, x, y, DESERTMINE_TILE);
     }
  
    if((tspecial & S_ROAD) ||(tspecial & S_RAILROAD))
     {
      tspecial_north=map_get_special(x, y-1);
      tspecial_east=map_get_special(x+1, y);
      tspecial_south=map_get_special(x, y+1);
      tspecial_west=map_get_special(x-1, y);

      if(tspecial & S_ROAD)
       {
	tileno=ROAD_TILES;
	tileno+=(tspecial_north&S_ROAD) ? 1 : 0;
	tileno+=(tspecial_east&S_ROAD)  ? 2 : 0;
	tileno+=(tspecial_south&S_ROAD) ? 4 : 0;
	tileno+=(tspecial_west&S_ROAD)  ? 8 : 0;
	draw_special(widget, x, y, tileno);
       }

      if(tspecial & S_RAILROAD)
       {
	tileno=RAIL_TILES;
	tileno+=(tspecial_north&S_RAILROAD) ? 1 : 0;
	tileno+=(tspecial_east&S_RAILROAD)  ? 2 : 0;
	tileno+=(tspecial_south&S_RAILROAD) ? 4 : 0;
	tileno+=(tspecial_west&S_RAILROAD)  ? 8 : 0;
	draw_special(widget, x, y, tileno);
       }
     } /* if(S_ROAD || S_RAILROAD) */

    if(tspecial & S_SPECIAL)
     {
      draw_special(widget, x, y, SPECIAL_TILES+ttype);
     }

    if(tspecial & S_HUT)
      draw_special(widget, x, y, HUT_TILE);

    if(tspecial & S_FORTRESS)
      draw_special(widget, x, y, FORTRESS_TILE);

    if(tspecial & S_POLLUTION)
      draw_special(widget, x, y, POLLUTION_TILE);
 
   } /* if(genlist_size) */
 } /* draw_tile()*/

  
/***************************************************************************
draw the specified special of the tile at(x,y)
****************************************************************************/
void draw_special(GtkWidget *widget, int x, int y, int tileno)
 {
  int tmpX, tmpY;

  tmpX=tileno%20;
  tmpY=tileno/20;
  
  gdk_gc_set_clip_origin(gc_tiles,(x-tmpX)*TILE_WIDTH,(y-tmpY)*TILE_HEIGHT);
  gdk_draw_pixmap(pix_screen, gc_tiles, pix_tiles,
		   tmpX*TILE_WIDTH, tmpY*TILE_HEIGHT,
		   x*TILE_WIDTH, y*TILE_HEIGHT, TILE_WIDTH, TILE_HEIGHT);
 } /* draw_special() */


/***************************************************************************
clear the specified rectangle of the map from specials
****************************************************************************/
void clear_rect_specials(int x0, int y0, int x1, int y1)
 {
  int i,j;
  
  if(x0>x1)
   {
    i=x0;x0=x1;x1=i;
   }
  if(y0>y1)
   {
    i=y0;y0=y1;y1=i;
   }

  for(i=x0;i<x1;i++)
    for(j=y0;j<y1;j++)
      map_get_tile(i,j)->special=S_NONE;
 } /* draw_special() */


/*
void draw_cities(GtkWidget *widget, struct genlist *list)
 {
  struct genlist_iterator iterator;
  struct city *pcity;

  genlist_iterator_init(&iterator, list, 0);
  for(; ITERATOR_PTR(iterator); ITERATOR_NEXT(iterator)) 
   {
    pcity=(struct city *)ITERATOR_PTR(iterator);
    gdk_draw_string(pixmap, city_font, widget->style->white_gc,
		    pcity->x*TILE_WIDTH+15-
                    (gdk_string_width(city_font, pcity->name)/2),  
		    pcity->y*TILE_HEIGHT+50, pcity->name);
   }
 }


void draw_city(GtkWidget *widget, struct city *pcity, struct genlist *list)
 {
  GdkFont *font;

  gdk_draw_pixmap(pixmap, widget->style->white_gc, pix_tiles,
		  game.players[pcity->owner].race*TILE_WIDTH, 14*TILE_HEIGHT,
		  pcity->x*TILE_WIDTH, pcity->y*TILE_HEIGHT,
		  TILE_WIDTH, TILE_HEIGHT);
  
  gdk_gc_set_clip_origin(gc_tiles,(pcity->x-((CITY_TILE)%20))*TILE_WIDTH, 
 			 (pcity->y -((CITY_TILE)/20))*TILE_HEIGHT);
  gdk_draw_pixmap(pixmap, gc_tiles, pix_tiles, (((city_got_citywalls(pcity)) ?
		  (CITY_WALLS_TILE):(CITY_TILE))%20)*TILE_WIDTH,
  		  ((CITY_TILE)/20)*TILE_HEIGHT,
		  pcity->x*TILE_WIDTH, pcity->y*TILE_HEIGHT,
		  TILE_WIDTH, TILE_HEIGHT);

  gdk_gc_set_clip_origin(gc_tiles,(pcity->x-(pcity->size%10))*TILE_WIDTH,
 			 (pcity->y-((NUMBER_TILES)/20))*TILE_HEIGHT);
  gdk_draw_pixmap(pixmap, gc_tiles, pix_tiles,
  		  (pcity->size%10)*TILE_WIDTH,((NUMBER_TILES)/20)*TILE_HEIGHT,
		  pcity->x*TILE_WIDTH, pcity->y*TILE_HEIGHT,
		  TILE_WIDTH, TILE_HEIGHT);

  if(pcity->size > 9)
   {
    gdk_gc_set_clip_origin(gc_tiles, (pcity->x-((NUMBER_MSD_TILES)%20)-
			   (pcity->size/10))*TILE_WIDTH,  
			   (pcity->y-((NUMBER_MSD_TILES)/20))*TILE_HEIGHT);
    gdk_draw_pixmap(pixmap, gc_tiles, pix_tiles,
		    (((NUMBER_MSD_TILES)%20)+(pcity->size/10))*TILE_WIDTH,
		    ((NUMBER_MSD_TILES)/20)*TILE_HEIGHT, 
		    pcity->x*TILE_WIDTH, pcity->y*TILE_HEIGHT,
		    TILE_WIDTH, TILE_HEIGHT);
   }

  genlist_insert(list, pcity, 0);

 }

*/
