#include "tools.h"
#include "civedit.h"
#include "draw.h"
#include "tiles.h"
#include "main.h"

#include "map.h"
#include "game.h"
#include "unit.h"

#include <stdio.h>

#define TABLE_WIDTH 3

/* Tools pixmaps */
#include "crayon.xpm"
#include "ligne.xpm"
#include "rectangle.xpm"
#include "unit.xpm"


int current_color=T_ARCTIC;
int tooltips_enabled=TRUE;
static GtkWidget *toolwin;
char labl[25];

Couleur couleurs[NB_COLORS] = 
 {
  {"Artic", T_ARCTIC, NULL, NULL, 0},
  {"Desert", T_DESERT, NULL, NULL, 0},
  {"Forest", T_FOREST, NULL, NULL, 0},
  {"Grassland", T_GRASSLAND, NULL, NULL, 0},
  {"Hills", T_HILLS, NULL, NULL, 0},
  {"Jungle", T_JUNGLE, NULL, NULL, 0},
  {"Mountains", T_MOUNTAINS, NULL, NULL, 0},
  {"Ocean", T_OCEAN, NULL, NULL, 0},
  {"Plains", T_PLAINS, NULL, NULL, 0},
  {"River", T_RIVER, NULL, NULL, 0},
  {"Swamp", T_SWAMP, NULL, NULL, 0},
  {"Tundra", T_TUNDRA, NULL, NULL, 0},
 };

Tool tools[TOOLS_NUMBER]=
 {
  {"Point", crayon_xpm, (GdkPixmap *) NULL, NULL, TOOLTYPE_SIMPLE,
   "Draw a single map tile",
   put_one_tile, (ToolFunction) NULL, NULL, 0},
  {"Line", ligne_xpm, (GdkPixmap *) NULL, NULL, TOOLTYPE_LINE,
   "Draw a line of map tiles",
   (ToolFunction) NULL, (ToolFunction) NULL, NULL, 0}, 
  {"Rectangle", rectangle_xpm, (GdkPixmap *) NULL, NULL, TOOLTYPE_AREA,
   "Draw a rectangle of map tiles",
   (ToolFunction) NULL, (ToolFunction) NULL, NULL, 0}, 
  {"Unit", unit_xpm, (GdkPixmap *) NULL, NULL, TOOLTYPE_SIMPLE,
   "Place the starting position for the currently selected race. ALL races must have a start position before those can be saved.",
   put_one_unit, (ToolFunction) NULL, NULL, 0}, 
 };


/****************************************************************************
set the drawing color (=terrain) to the currently selected one
*****************************************************************************/
void set_color(GtkWidget *widget, gpointer color)
 {
  int i;

  current_color=couleurs[(int) color].parametre;

  /* press the current button and unpress all others */
  for (i=0; i<NB_COLORS; i++)
   {
    gtk_signal_handler_block(GTK_OBJECT(couleurs[i].button), couleurs[i].id);
    if (i!=(int) color)
     {
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(couleurs[i].button), FALSE);
     } else {
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(couleurs[i].button), TRUE);
     }
    gtk_signal_handler_unblock(GTK_OBJECT(couleurs[i].button),couleurs[i].id);
   }
  sprintf(labl,"%s ",couleurs[(int) color].name);
  terrain_label(labl);
 } /* set_color() */


/****************************************************************************
create the toggle buttons in the palette table
*****************************************************************************/
void make_colors (GtkWidget *table)
 {
  int i;

  for (i=0; i<NB_COLORS; i++)
   {
    couleurs[i].button=gtk_toggle_button_new_with_label(couleurs[i].name);
    gtk_table_attach(GTK_TABLE(table), couleurs[i].button,
		     i%TABLE_WIDTH, i%TABLE_WIDTH+1,
		     i/TABLE_WIDTH, i/TABLE_WIDTH+1,
		     GTK_FILL, GTK_FILL, 0, 0);
    couleurs[i].id=gtk_signal_connect(GTK_OBJECT(couleurs[i].button),
				      "toggled",
				      GTK_SIGNAL_FUNC(set_color),
                                      (gpointer) i);
    gtk_widget_show(couleurs[i].button);
   }

  current_color=couleurs[0].parametre;
  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(couleurs[0].button), TRUE);  
 } /* make_colors() */


/****************************************************************************
create the palette table
*****************************************************************************/
GtkWidget *create_palette ()
 {
  GtkWidget * table;

  table=gtk_table_new(TABLE_WIDTH, (NB_COLORS+TABLE_WIDTH-1)/TABLE_WIDTH,
                      TRUE);

  make_colors(table);

  return table;
 } /* create_palette */


/***************************************************************************
fill the tools[]-array with the missing values: pixmaps, buttons, ids
***************************************************************************/
void create_tools(GtkWidget *win, GtkWidget *parent)
 {
  int i;
  GtkWidget *pixmap;

  tooltips=gtk_tooltips_new();

  /* create the button label pixmaps from the xpm's */
  for (i=0; i<TOOLS_NUMBER; i++)
   {
    if (tools[i].pixmap_data != NULL)
     {
      tools[i].pixmap=gdk_pixmap_create_from_xpm_d(
        win->window, &tools[i].mask, &white, tools[i].pixmap_data);
     }
    else
     {
      tools[i].pixmap=NULL;
     }
   }
  
  for (i=0; i<TOOLS_NUMBER; i++)
   {
    tools[i].button=gtk_toggle_button_new();
    if (tools[i].pixmap != NULL)
     {
      pixmap=gtk_pixmap_new(tools[i].pixmap, tools[i].mask);
      gtk_container_add(GTK_CONTAINER(tools[i].button), pixmap);
      gtk_widget_show(pixmap);
     }

    gtk_tooltips_set_tip(tooltips, tools[i].button, tools[i].tiptext, NULL);

    gtk_box_pack_start(GTK_BOX(parent), tools[i].button, TRUE, TRUE, 5);
    tools[i].id=gtk_signal_connect (GTK_OBJECT(tools[i].button),
				    "toggled",
				    GTK_SIGNAL_FUNC(tool_toggled),
				    (gpointer) i); 
    gtk_widget_show (tools[i].button);
   }

  current_tool=&tools[0];
  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(tools[0].button), TRUE);  
 } /* create_tools() */


/***************************************************************************
create the Toolbar window
***************************************************************************/
void create_tools_palette()
 {
  GtkWidget *vbox;
  GtkWidget *hbox;
  GtkWidget *palette;
  GtkWidget *but;

  toolwin=gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(toolwin), "Toolbar");
  gtk_widget_set_usize(GTK_WIDGET(toolwin),210,200);
  gtk_window_set_policy(GTK_WINDOW(toolwin), FALSE, FALSE, FALSE);
  gtk_signal_connect(GTK_OBJECT(toolwin), "delete_event",
		     GTK_SIGNAL_FUNC(destroyWindow), toolwin);
  vbox=gtk_vbox_new(FALSE, 10);
  gtk_container_add(GTK_CONTAINER(toolwin), vbox);
  gtk_widget_show(vbox);
  gtk_widget_show(toolwin);

  /* tool button creation */
  hbox=gtk_hbox_new(TRUE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 10);
  create_tools(toolwin, hbox);
  gtk_widget_show(hbox);

  /* palette creation */
  hbox=gtk_hbox_new(TRUE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 10);
  palette=create_palette();
  gtk_box_pack_start(GTK_BOX(hbox), palette, FALSE, TRUE, 10);
  gtk_widget_show(palette);
  gtk_widget_show(hbox);
  
  /* and finally the close button*/
  hbox=gtk_hbox_new(TRUE, 0);
  gtk_box_pack_end(GTK_BOX(vbox), hbox, FALSE, TRUE, 5);
  but=gtk_button_new_with_label ("Close");
  gtk_signal_connect(GTK_OBJECT(but), "clicked",
			GTK_SIGNAL_FUNC(delete), NULL);
  gtk_box_pack_start (GTK_BOX(hbox), but, TRUE, FALSE, 0);
  gtk_widget_show (but);
  gtk_widget_show (hbox);
 } /* create_tools_palette() */


/*****************************************************************************
change the color, i.e. terrain, of the tile at (x,y)
*****************************************************************************/
void put_one_tile(GtkWidget *widget, int x, int y)
 {
  struct tile *ptile;
  int i, j;

  for(i=0;i<R_LAST;i++)
    if((map.start_positions[i].x==x) &&
       (map.start_positions[i].y==y) &&
       (current_color!=T_GRASSLAND) &&
       (current_color!=T_PLAINS) &&
       (current_color!=T_RIVER) )
      return;

  ptile=map_get_tile(x, y);
  ptile->terrain=current_color;

  /* may be buggy at the world wrap?! */
  for (i=-1; i<=1; i++)
   {
    for (j=-1; j<=1; j++)
      draw_tile(widget, x+i, y+j, NULL); 
   }
  refresh_screen();
}


/*****************************************************************************
change the starting position of the current race to (x,y)
*****************************************************************************/
void put_one_unit(GtkWidget *widget, int x, int y)
 {
  int i, j;
  int xo, yo;

  /* don't place starting positions on unfriendly terrain */
  if (map_get_terrain(x, y)!=T_PLAINS &&
      map_get_terrain(x, y)!=T_GRASSLAND &&
      map_get_terrain(x, y)!=T_RIVER)
    return;

  /* delete the settler unit picture from the old starting position */
  xo=map.start_positions[country].x;
  yo=map.start_positions[country].y;
  for (i=-1; i<=1; i++)
    {
      for (j=-1; j<=1; j++)
	draw_tile (widget, xo+i, yo+j, NULL); 
    }

  /* enter the new starting positions into the map structure and redraw */
  map.start_positions[country].x=x;
  map.start_positions[country].y=y;
  game.scenario=2; /* default to tilemap + startpositions */
  /* default to no start positions if at least 1 is missing */
  for(i=0;i<R_LAST;i++)
    if((map.start_positions[i].x==0) &&
       (map.start_positions[i].y==0))
      game.scenario=3;

  draw_units(widget,0,0,map.xsize*TILE_WIDTH,map.ysize*TILE_HEIGHT);
  refresh_screen();
 } /* put_one_unit() */


/*****************************************************************************
handle the toggle buttons' toggle events
*****************************************************************************/
void tool_toggled(GtkWidget *widget, gpointer tool)
 {
  int i,j;

  j=(int) tool;
  current_tool=&tools[j];

  /* press the current button and unpress all others*/
  for (i=0; i<TOOLS_NUMBER; i++)
   {
    gtk_signal_handler_block(GTK_OBJECT(tools[i].button), tools[i].id); 
    if (i!=j)
     {
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(tools[i].button), FALSE);  
     } else {
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(tools[i].button), TRUE);  
     }
     gtk_signal_handler_unblock(GTK_OBJECT(tools[i].button), tools[i].id);
   }

  for(i=0;i<NB_COLORS;i++)
   {
    if(couleurs[i].button!=NULL)
      if(j==3)
        gtk_widget_set_sensitive(couleurs[i].button,FALSE);
      else
        gtk_widget_set_sensitive(couleurs[i].button,TRUE);
   }
 } /* tool_toggled() */


/*****************************************************************************
kill the toplevel window upon wm's window delete or pressed close button
*****************************************************************************/
void delete(GtkWidget *widget, gpointer data)
 {
  gtk_widget_hide(toolwin);
 } /* delete() */

int destroyWindow(GtkWidget *widget, GdkEvent *ev, gpointer removedWidget)
 {
  gtk_widget_hide(widget);
  return TRUE;
 } /* destroyWindow() */


/*****************************************************************************
show the toolbox window
*****************************************************************************/
void show_tools(void)
 {
   gtk_widget_show(GTK_WIDGET(toolwin));
 } /* show_tools() */


/*****************************************************************************
toggle tooltips on and off
*****************************************************************************/
void show_tips(void)
 {
   if(tooltips_enabled)
    {
     gtk_tooltips_disable(tooltips);
     tooltips_enabled=FALSE;
    }
   else
    {
     gtk_tooltips_enable(tooltips);
     tooltips_enabled=TRUE;
    }
 } /* show_tips() */
